#!/bin/bash
# Installation/configuration diagnostic script
# Smart Update Tools ESXi for Gen8/Gen9
#

PASS='\t\t [ \e[1;32mPASS\e[0m ]'
FAIL='\t\t [ \e[1;31mFAIL\e[0m ]'
UNKNOWN='\t\t [ \e[1;31mUNKNOWN\e[0m ]'

IsServiceRunning=0
ReturnCode=1
TempFile="/var/tmp/output.txt"
OVStatus=""
VCenterStatus=""
OVURL="Management Host URL"
VCENTERURL="vCenter Server"
STATUS_UNKNOWN="Unknown"
STATUS_SUCCESS="ConnectionSuccess" 

#This function will get the available free disk space 
function get_free_disk_space() {
   echo -ne 'Checking required free disk space...'
   eval Memory=$(df -P /var/tmp | sed -n "2,1p" |awk '{print $4}')
   Memory=`expr $Memory / $((1024 * 1024))`
   if [ $Memory -ge 4 ]; then
      echo -e $PASS
   else
      echo -e $FAIL
      echo -e '\t[\e[1;31mWARNING\e[0m]:: Available free disk space = '$Memory GB.
      echo -e '\t[\e[1;33mResolution\e[0m]:: Clean up disk space, minimum 4 GB of free disk space is required where stagingdirectory is created.' 
   fi
}

#This function will get the sutesxi prerequisite information
function get_sutesxi_prerequisite_info() {
   echo -ne 'Checking SUTESXi prerequisites info...'
   FAILED_DEPENDENCY=0
   Missing_packages=''
   sut_rpm_requires=$(rpm -q sutesxi --queryformat '[%{REQUIRES}\n]'>$TempFile 2>/dev/null)
   while IFS= read -r line
   do
      if [[ "$line" == rpmlib* ]]; then
         package_count=$(rpm --showrc | grep rpmlib | grep $line | wc -l  2>/dev/null)
         if [[ $package_count -le 0 ]]; then
            FAILED_DEPENDENCY=1
            Missing_packages="$Missing_packages $line"
         fi
      elif [[ "$line" == "module-init-tools" ||  "$line" == procps ]]; then
         rpmpackage=$(rpm -q --whatprovides $line 2>/dev/null)
         rpmprovides=$(rpm -q --provides $rpmpackage 2>/dev/null)
         returnC=$?
         if [[ $returnC -ne 0 ]]; then
	    FAILED_DEPENDENCY=1
            Missing_packages="$Missing_packages $line"
         fi
     elif [[ "$line" == "shutdown" ]]; then
         if ! [[ -f /sbin/shutdown || -f /usr/sbin/shutdown  ]]; then
	    FAILED_DEPENDENCY=1
            Missing_packages="$Missing_packages $line"
         fi
     elif [[ "$line" == "kill" ]]; then
         if ! [[ -f /bin/kill || -f /usr/bin/kill ]]; then
	    FAILED_DEPENDENCY=1
            Missing_packages="$Missing_packages $line"
         fi
     elif [[ "$line" == */bin* || "$line" == */sbin* ]]; then
         if ! [[ -f $line ]]; then
	    FAILED_DEPENDENCY=1
            Missing_packages="$Missing_packages $line"
         fi
     else
         data=$(rpm -q --provides $line >/dev/null)
         returnC=$?
         if [[ $returnC -ne 0 ]]; then
	    FAILED_DEPENDENCY=1
            Missing_packages="$Missing_packages $line"
         fi
     fi
   done <"$TempFile"
  
   if [ $FAILED_DEPENDENCY -eq 0 ]; then
      echo -e $PASS 
   else
      echo -e $FAIL
      echo -e '\t[\e[1;31mWARNING\e[0m]::Missing packages are ='$Missing_packages
      echo -e '\t[\e[1;33mResolution\e[0m]:: Please ensure above packages are installed and available for sutesxi service to work normally.'
   fi
}

#This function will get the SUM prerequisite information
function get_sum_prerequisite_info() {
   echo -ne 'Checking SUM prerequisites info...'
   FAILED_DEPENDENCY=0
   MissingTools=''
   
   #Check the commands present or not
   #1. sed
   commandName="sed"
   which $commandName  >/dev/null 2>&1
   if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $commandName"
      FAILED_DEPENDENCY=1
   fi       
   #2. gawk
   commandName="gawk"
   which $commandName  >/dev/null 2>&1
   if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $commandName"
      FAILED_DEPENDENCY=1
   fi       
   #3. grep
   commandName="grep"
   which $commandName  >/dev/null 2>&1
   if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $commandName"
      FAILED_DEPENDENCY=1
   fi       
   #4. cat
   commandName="cat"
   which $commandName  >/dev/null 2>&1
   if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $commandName"
      FAILED_DEPENDENCY=1
   fi       
   #5. nohup
   commandName="nohup"
   which $commandName  >/dev/null 2>&1
   if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $commandName"
      FAILED_DEPENDENCY=1
   fi       
   #6. dirname
   commandName="dirname"
   which $commandName  >/dev/null 2>&1
   if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $commandName"
      FAILED_DEPENDENCY=1
   fi       
   #7. cut
   commandName="cut"
   which $commandName  >/dev/null 2>&1
   if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $commandName"
      FAILED_DEPENDENCY=1
   fi       
   #8. ls
   commandName="ls"
   which $commandName  >/dev/null 2>&1
   if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $commandName"
      FAILED_DEPENDENCY=1
   fi       
   #9. uname
   commandName="uname"
   which $commandName  >/dev/null 2>&1
   if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $commandName"
      FAILED_DEPENDENCY=1
   fi       
   #10. df
   commandName="df"
   which $commandName  >/dev/null 2>&1
   if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $commandName"
      FAILED_DEPENDENCY=1
   fi       
   #11. lsmod
   commandName="lsmod"
   which $commandName  >/dev/null 2>&1
   if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $commandName"
      FAILED_DEPENDENCY=1
   fi       
   #12. lspci
   commandName="lspci"
   which $commandName  >/dev/null 2>&1 
   if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $commandName"
      FAILED_DEPENDENCY=1
   fi       
   #13. awk 
   commandName="awk"
   which $commandName  >/dev/null 2>&1 
   if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $commandName"
      FAILED_DEPENDENCY=1
   fi       
   #14. ps
   commandName="ps"
   which $commandName  >/dev/null 2>&1 
   if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $commandName"
      FAILED_DEPENDENCY=1
   fi       
   #15. strings
   commandName="strings"
   which $commandName  >/dev/null 2>&1 
   if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $commandName"
      FAILED_DEPENDENCY=1
   fi       
   #16. unzip 
   commandName="unzip"
   which $commandName  >/dev/null 2>&1 
   if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $commandName"
      FAILED_DEPENDENCY=1
   fi       
   #17. cpio 
   commandName="cpio"
   which $commandName  >/dev/null 2>&1 
   if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $commandName"
      FAILED_DEPENDENCY=1
   fi       
   #18. rpm2cpio
   commandName="rpm2cpio"
   which $commandName  >/dev/null 2>&1 
   if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $commandName"
      FAILED_DEPENDENCY=1
   fi       
   #19. ldconfig
   commandName="ldconfig"
   which $commandName  >/dev/null 2>&1 
   if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $commandName"
      FAILED_DEPENDENCY=1
   fi       
   #20. ldd
   commandName="ldd"
   which $commandName  >/dev/null 2>&1 
   if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $commandName"
      FAILED_DEPENDENCY=1
   fi       
   #21. sudo
   commandName="sudo"
   which $commandName  >/dev/null 2>&1 
   if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $commandName"
      FAILED_DEPENDENCY=1
   fi       
   #22. su
   commandName="su"
   which $commandName  >/dev/null 2>&1 
   if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $commandName"
      FAILED_DEPENDENCY=1
   fi       
   #23. chown
   commandName="chown"
   which $commandName  >/dev/null 2>&1 
   if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $commandName"
      FAILED_DEPENDENCY=1
   fi       
   #24. reboot
   commandName="reboot"
   which $commandName  >/dev/null 2>&1 
   if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $commandName"
      FAILED_DEPENDENCY=1
   fi       
   #25. stat
   commandName="stat"
   which $commandName  >/dev/null 2>&1 
   if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $commandName"
      FAILED_DEPENDENCY=1
   fi       
   #26. id
   commandName="id"
   which $commandName  >/dev/null 2>&1 
   if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $commandName"
      FAILED_DEPENDENCY=1
   fi       
   #27. wc
   commandName="wc"
   which $commandName  >/dev/null 2>&1 
   if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $commandName"
      FAILED_DEPENDENCY=1
   fi       

  #Check the libs are present or not
  #1. libcrypt 
  libName="libcrypt"
  ldconfig -p | grep $libName >/dev/null 2>&1
  if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $libName"
      FAILED_DEPENDENCY=1
  fi
  #2. libpthread 
  libName="libpthread"
  ldconfig -p | grep $libName >/dev/null 2>&1
  if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $libName"
      FAILED_DEPENDENCY=1
  fi
  #3. libz 
  libName="libz"
  ldconfig -p | grep $libName >/dev/null 2>&1
  if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $libName"
      FAILED_DEPENDENCY=1
  fi
  #4. libdl
  libName="libdl"
  ldconfig -p | grep $libName >/dev/null 2>&1
  if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $libName"
      FAILED_DEPENDENCY=1
  fi
  #5. librt
  libName="librt"
  ldconfig -p | grep $libName >/dev/null 2>&1
  if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $libName"
      FAILED_DEPENDENCY=1
  fi
  #6. libstdc++
  libName="libstdc++"
  ldconfig -p | grep $libName >/dev/null 2>&1
  if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $libName"
      FAILED_DEPENDENCY=1
  fi
  #7. libm
  libName="libm"
  ldconfig -p | grep $libName >/dev/null 2>&1
  if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $libName"
      FAILED_DEPENDENCY=1
  fi
  #8. libgcc_s
  libName="libgcc_s"
  ldconfig -p | grep $libName >/dev/null 2>&1
  if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $libName"
      FAILED_DEPENDENCY=1
  fi
  #9. libc
  libName="libc"
  ldconfig -p | grep $libName >/dev/null 2>&1
  if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $libName"
      FAILED_DEPENDENCY=1
  fi
  #10. ld-linux-x86-64
  libName="ld-linux-x86-64"
  ldconfig -p | grep $libName >/dev/null 2>&1
  if [ $? -ne 0 ]; then
      MissingTools="$MissingTools $libName"
      FAILED_DEPENDENCY=1
  fi

  if [ $FAILED_DEPENDENCY -eq 0 ]; then
     echo -e $PASS 
  else
     echo -e $FAIL 
     echo -e '\t[\e[1;31mWARNING\e[0m]::Missing required utilities and libs = '$MissingTools
     echo -e '\t[\e[1;33mResolution\e[0m]:: Please ensure above utilites/libs  are available for SUM to work normally.'
  fi

}

#This function will check the required ports are open or not
function get_required_port_info() {
   echo -ne 'Checking required open ports status...'
   port_free=1
   used_ports=""
   ListenPorts=$(netstat -lnp | grep LISTEN | grep "\\b.*:63001\\b")
   if [[ $ListenPorts != "" ]]; then
      processId=$(echo $ListenPorts | awk '{print $7}' | cut -d / -f1)
      programName=$(ps -ef | grep -v grep | grep $processId |  awk '{print $8}')
      if ! [[ "$programName" == /var/tmp/sutesxi* ]]; then
         port_free=0
         used_ports="63001"
      fi
   fi
   
   ListenPorts=$(netstat -lnp | grep LISTEN | grep "\\b.*:63002\\b")
   if [[ $ListenPorts != "" ]]; then
      processId=$(echo $ListenPorts | awk '{print $7}' | cut -d / -f1)
      programName=$(ps -ef | grep -v grep | grep $processId |  awk '{print $8}')
      if ! [[ "$programName" == /var/tmp/sutesxi* ]]; then
         used_ports="$used_ports 63002"
         port_free=0
      fi
   fi

   ListenPorts=$(netstat -ln | grep LISTEN | grep "\\b.*:5898\\b")
   if [[ $ListenPorts != "" ]]; then
      used_ports="$used_ports 5898"
      port_free=0
   fi

   ListenPorts=$(netstat -ln | grep LISTEN | grep "\\b.*:443\\b")
   if [[ $ListenPorts != "" ]]; then
      used_ports="$used_ports 443"
      port_free=0
   fi
   
   ListenPorts=$(netstat -ln | grep LISTEN | grep "\\b.*:80\\b")
   if [[ $ListenPorts != "" ]]; then
      used_ports="$used_ports $80"
      port_free=0
   fi

   if [ $port_free -eq 1 ];then
      echo -e $PASS
   else
      echo -e $FAIL
      echo -e '\t[\e[1;31mWARNING\e[0m]::Occupied ports are ='$used_ports
      echo -e '\t[\e[1;33mResolution\e[0m]:: Please ensure above ports are not blocked by firwall, netheir occupied by any other program.'
   fi
}

#This function will show a progress bar during any sutesxi command execution
function show_progress_bar() {
 #$1=command name
 if [ "$1" = "start" ]; then
    waitInSeconds=0.8
 else
    waitInSeconds=0.4
 fi

 Progress=''
 PROC_ID=$!
 Progress=''
 Tabs='\t\t\t\t\t\t'
 NumOfHash=0
 Percentage=0
 Timeedout=0
 ElapsedTime=0.0
 
 while kill -0 "$PROC_ID" >/dev/null 2>&1; do
    echo -ne '\r'$Progress$Tabs [$Percentage%]
    if [ "$Percentage" -eq 99 ]; then
       NumOfHash=`expr $NumOfHash - 2`
       if [ ${ElapsedTime%.*} -ge 300 ]; then
         Timeedout=1
         echo -ne '\n'
         echo -e '\t[\e[1;31mWARNING\e[0m]::Command could not be completed within expected time of 5 minutes.'
         echo -e '\t[\e[1;33mResolution\e[0m]:: Please stop and restart the service and try again.' 
         break
       else
          eval ElapsedTime=`echo $ElapsedTime + $waitInSeconds|bc`
       fi
    elif [ "$Percentage" -ge 90 ]; then
       Percentage=`expr $Percentage + 1`
       Progress="#"$Progress 
       NumOfHash=`expr $NumOfHash + 1`
    elif [ "$Percentage" -ge 70 ]; then
       Percentage=`expr $Percentage + 4`
       Progress="##"$Progress 
       NumOfHash=`expr $NumOfHash + 2`
    elif [ "$Percentage" -ge 50 ]; then
       Percentage=`expr $Percentage + 5`
       Progress="##"$Progress 
       NumOfHash=`expr $NumOfHash + 2`
    else
       Progress="##"$Progress 
       Percentage=`expr $Percentage + 5`
       NumOfHash=`expr $NumOfHash + 2`
   fi
       sleep `echo $waitInSeconds + 0|bc` 
       eval ElapsedTime=`echo $ElapsedTime + $waitInSeconds|bc`
   
   if [ "$NumOfHash" -ge 8 ]; then
      Tabs=${Tabs%?};
      Tabs=${Tabs%?};
      NumOfHash=0
   fi
 done
if [ $Timeedout -eq 0 ]; then
 wait $PROC_ID 
 ReturnCode=$?
 while [ "$Percentage" -le 99 ]; do
    Percentage=`expr $Percentage + 1`
    echo -ne '\r'$Progress$Tabs [$Percentage%]
    Progress="#"$Progress 
    if [ "$NumOfHash" -ge 8 ]; then
       Tabs=${Tabs%?};
       Tabs=${Tabs%?};
       NumOfHash=0
    fi
 done
fi
 echo -ne '\n'
}

#This function will check if the sutesxi service is running or not
function check_if_service_is_running() {
PROCESS_COUNT=$(ps -efww | grep "sutesxiservice" | grep -v "grep" | wc -l)
if [[ $PROCESS_COUNT -eq 0 ]]; then
   validinput=0;
   echo -e  '\e[1;33mSUTESXi service must be running to get the OneView and vCenter information.\e[0m'
   read -p "Do you want to start now ? [Y/n] : " proceed
   while [ $validinput -ne 1 ]; do
      if [[ "$proceed" == "Y" || "$proceed" == "y" ]]; then
         validinput=1;
         echo -e 'Starting sutesxi service...'
         sutesxi -start > $TempFile 2>&1 &
         show_progress_bar start 
         if [[ $ReturnCode -eq 0 ]]; then
            PROCESS_COUNT=$(ps -efww | grep "sutesxiservice" | grep -v "grep" | wc -l)
            if [[ $PROCESS_NUM -eq 0 ]]; then
               echo -e '[\e[1;32mSUCCESS\e[0m]::Successfully started the SUTESXi Service.'
               IsServiceRunning=1
            fi
        fi
      elif [[ "$proceed" == "N" || "$proceed" == "n" ]]; then
         validinput=1;
         echo -e '[\e[1;33mINFO\e[0m]::OneView and vCenter information could not be retrieved.'
      else
         read -p "Please enter Y or n : " proceed
      fi
   done
else
   IsServiceRunning=1
fi
}

#This function will set the vcenterhost
function set_vCenter_host() {
#If vCenter connection is failed prompt user if he wants to set vCenter now
   echo -ne 'Checking vCenter configuration...'
   echo -e $VCenterStatus
   if [[ "$VCenterStatus" == "$FAIL" ]]; then
   echo -e 'vCenter host is not configured.'
      validinput=0;
      read -p "Do you want to configure vCenter now ? [Y/n] " answer
      while [ $validinput -ne 1 ]; do
         if [[ "$answer" == "Y" || "$answer" == "y" ]]; then
            validinput=1;
            read -p "Enter vCenter ip address or FQDN name: "  ip
            read -p "Enter vCenter user name: "  user
            read -s -p "Enter vCenter password: "  password
            echo -ne '\n'
   	    echo -e 'Setting up vCenter host...'
            sutesxi -set vcenterhost=$ip vcenterusername=$user vcenterpassword=$password > $TempFile  2>&1 &
            show_progress_bar 

            if [[ $ReturnCode -eq 0 ]]; then
               echo -e '[\e[1;32mSUCCESS\e[0m]::Successfully configured the vCenter.'
            else
	       echo -e '[\e[1;31mERROR\e[0m]::Unable to configure the vCenter, please check the connectivitiy and credentials then retry again.'
               read -p "Do you want to retry ? [Y/n] " answer
               validinput=0;
	    fi
       elif [[ "$answer" == "N" || "$answer" == "n" ]]; then
          validinput=1;
       else
          read -p "Please enter Y or n : " answer
       fi
    done
  fi
}

#This function will set the oneview host
function set_OneView_host() {
#If OneView connection is failed prompt user if he wants to set OneView  now
   echo -ne 'Checking OneView configuration...'
   echo -e $OVStatus
   if [[ "$OVStatus" == "$FAIL" ]]; then
   echo -e 'OneView host is not configured.'
      validinput=0;
      read -p "Do you want to configure the Management host(OneView) now ? [Y/n] " answer
      while [ $validinput -ne 1 ]; do
         if [[ "$answer" == "Y" || "$answer" == "y" ]]; then
            validinput=1;
            read -p "Enter management host ip address or FQDN name: "  ip
            read -p "Enter management host user name: "  user
            read -s -p "Enter management host password: "  password
            echo -ne '\n'
	    echo -e 'Setting up OneView host...'
            sutesxi -set managementhost=$ip managementhostusername=$user managementhostpassword=$password > $TempFile  2>&1 &
            show_progress_bar
	    if [[ $ReturnCode -eq 0 ]]; then
	       echo -e '[\e[1;32mSUCCESS\e[0m]::Successfully configured the Management host.'
            else
	       echo -e '[\e[1;31mERROR\e[0m]::Unable to configure the Management host, please check the connectivitiy and credentials then retry again.'
               read -p "Do you want to retry ? [Y/n] " answer
               validinput=0;
	    fi
        elif [[ "$answer" == "N" || "$answer" == "n" ]]; then
	   validinput=1;
	else
	   read -p "Please enter Y or n : " answer
        fi
   done
fi
}

#This function will read status command output from the file and parse it
function read_status_output() {
   OVStatus=$UNKNOWN
   VCenterStatus=$UNKNOWN
   while IFS= read -r line; do
       if [[ "$line" == *$OVURL* ]]; then
          IFS= read -r line	
          OVConnection="Management Host Status"
          if [[ "$line" == *$OVConnection* ]]; then
             OVConnectionStatus=$(echo $line | cut -d ":" -f 2 | sed 's/ //g')
	     if [ "$OVConnectionStatus" = "$STATUS_SUCCESS" ]; then
                OVStatus=$PASS
	     else
                OVStatus=$FAIL
	     fi
          fi
       elif [[ "$line" == *$VCENTERURL* ]]; then
          IFS= read -r line
          VCConnection="vCenter Server Status"
          if [[ "$line" == *$VCConnection* ]]; then
             VCConnectionStatus=$(echo $line | cut -d ":" -f 2 | sed 's/ //g')
             if [ "$VCConnectionStatus" = "$STATUS_SUCCESS" ]; then
	        VCenterStatus=$PASS
             else
                VCenterStatus=$FAIL
             fi
         fi
      fi
   done <"$TempFile"
}

###################################################################################
#                                                                                 #
#                           main starts here                                      #
#                                                                                 #
###################################################################################

 # Display copyright
 echo ""
 echo "Copyright (C) 2020 Hewlett Packard Enterprise Development LP."
 echo "-----------------------------------------------------------------------"

 #1. Get free space info
 get_free_disk_space

 #2. Get open port info
 get_required_port_info

 #3. Get sutesxi prerequisite info
 get_sutesxi_prerequisite_info
 
 #4. Get sum prerequisite info
 get_sum_prerequisite_info

 #5. Get vCenter and OneView information
 #Check if the service is running or not, if not running prompt user if he wants to start the service to get OneView and vCenter info
 check_if_service_is_running

 #Get OneView and vCenter information by running sutesxi -status command
 if [[ $IsServiceRunning -eq 1 ]]; then
    echo -e 'Collecting OneView and vCenter information...'
    sutesxi -status > $TempFile  2>&1 &
    show_progress_bar
    read_status_output
    set_OneView_host
    set_vCenter_host
 fi
 
 #Remove the temp file created
 rm -f $TempFile


